import 'package:flutter/material.dart';

import 'package:flutter_html_view/flutter_html_view.dart';
import 'package:cached_network_image/cached_network_image.dart';

import 'package:spartathlon_app/models/BlogEntry.dart';
import 'package:spartathlon_app/util/Utils.dart';

/// Blog Details View
///
/// Displays a blog entry (race report) with the header image at the top, followed by the report itself.

class BlogFullView extends StatefulWidget {
  BlogFullView({Key key, this.item}) : super(key: key);
  final BlogEntry item;

  @override
  _BlogFullViewState createState() => _BlogFullViewState();
}

class _BlogFullViewState extends State<BlogFullView> {
  ScrollController _scrollController;
  double kExpandedHeight;

  @override
  void initState() {
    super.initState();
    _scrollController = ScrollController()..addListener(() => setState(() {}));
    kExpandedHeight = 1;
  }

  bool isCollapsed() {
    return _scrollController.hasClients &&
        _scrollController.offset > kExpandedHeight - 1.7 * kToolbarHeight;
  }

  @override
  Widget build(BuildContext context) {
    kExpandedHeight = MediaQuery.of(context).size.height / 2;
    double width = MediaQuery.of(context).size.width;

    return CustomScrollView(
      controller: _scrollController,
      slivers: [
        SliverAppBar(
          backgroundColor: Theme.of(context).primaryColor,
          expandedHeight: kExpandedHeight,
          floating: false,
          pinned: true,
          actions: [
            PopupMenuButton(
              itemBuilder: (BuildContext context) {
                return [
                  PopupMenuItem(
                    child:
                        Container(width: 120.0, child: Text('Open in browser')),
                    value: widget.item.url,
                  )
                ];
              },
              onSelected: (String url) => launchUrl(url),
            )
          ],
          flexibleSpace: FlexibleSpaceBar(
              // TODO align title text to the far left
              // tracked in https://github.com/flutter/flutter/issues/11951
              title: ConstrainedBox(
                constraints:
                    BoxConstraints(maxWidth: width - 190, maxHeight: 36.0),
                child: Text(
                  widget.item.headline,
                  overflow: TextOverflow.visible,
                ),
              ),
              titlePadding: isCollapsed() // align to the far left if expanded
                  ? EdgeInsets.only(bottom: 16.0, left: 56.0)
                  : EdgeInsets.only(bottom: 16.0, left: 16.0),
              background: Stack(
                fit: StackFit.expand,
                children: [
                  CachedNetworkImage(
                    imageUrl: widget.item.imageUrl,
                    fit: BoxFit.cover,
                    // TODO Upgrade to cached_network_image 0.7.0
                    // Workaround for https://github.com/renefloor/flutter_cached_network_image/issues/134
                    width: double.infinity,
                    height: double.infinity,
                    alignment: Alignment(0, 0.5),
                  ),
                ],
              )),
        ),
        // The flutter_html_view package has not been well-maintained in
        // the past (especially wrt to updating to newer dependencies.
        // Also, because it parses the entire HTML text, it lags for larger texts.
        // TODO Consider replacing it by something else
        SliverToBoxAdapter(
          child: Material(
            child: Container(
              padding: EdgeInsets.all(12.0),
              child: HtmlView(
                data: widget.item.fullText,
                scrollable: false,
              ),
            ),
          ),
        ),
      ],
    );
  }
}
